# DAHDI-Linux with DKMS

**Link to the full documentation:** https://gitlab.com/avencall/randd-doc/wikis/dkms

## Link with dahdi official repo

- This project contains a copy (yes that's not perfect) of the [dahdi official repo](git://git.asterisk.org/dahdi/linux).
- For tag 2022.04.00-rc it corresponds to commit `5c840cf` in branch `next` of the project.
  - we had to use this branch to be able to have dahdi for Debian 11 which has a kernel > 5.6.0
  - see https://community.asterisk.org/t/dahdi-fails-fatal-error-linux-pci-aspm-h-no-such-file-or-directory/83561

## How to update 

If you have to update this project (basically if you need to have a new dahdi version):
1. clone *our* repo: git clone git@gitlab.com:xivo.solutions/dahdi-linux.git
2. clone official dahdi repo: git clone git://git.asterisk.org/dahdi/linux official-dahdi-linux
3. go to the official dahdi repo: cd official-dahdi-linux
4. go to the tag/commit/branch that you want in the official repo: git checkout <tag>
5. then copy the official repo content into our repo: cp official-dahdi-linux/* ~/dev/xivo.solutions/dahdi-linux/
6. in our repo bump the `.version` file to 
   - `<dahdi version>` (e.g. 3.2.0) - if dahdi version was updated
   - or `<dahdi version>+<xivo version>` (e.g. 3.1.0+xivo2) - if dahdi version was not updated
7. in our repo bump the `debian/changelog` with the new version in `dahdi-linux-dkms-mkdsc/debian/changelog` (with same version as in `.version`)
8. then create a MR and try to build this branch

## Introduction: XiVO and DKMS
This folder contains XiVO's version of DAHDI-Linux. It has been restructured in order to be able to work with DKMS.

DKMS is a framework which allows automatic rebuilding of modules when a new kernel is installed. Getting those modules to build with DKMS allows them to build themselves automatically at every kernel update, thus saving time (as long as the modules are compatible with the current kernel version).

## Build requirements

- Debian 9
- devscripts: 2.17.6+deb9u2 (stretch/main)
- dpkg-dev: 1.18.25 (stretch/main)
- dkms: 2.6.1-4 (buster/main) - can be installed by temporarily adding "buster" source
- xivo-build-tools: branch debian10

## Instructions
Here is the list of commands to type to generate a debian package for dahdi-linux 3.1.0 with dkms:
- Note 1: those commands have been typed as root, you might need to use sudo.
- Note 2: XiVO’s patches have already been applied - you do not need to bother with them.

Install DKMS version 2.6 or newer (if this link doesn’t work, take another on https://pkgs.org/download/dkms ).
  - Do not install DKMS with ```sudo apt-get install dkms```. This will install an old version of DKMS on your computer that is not compatible with the DAHDI-Linux package.

```bash
cd /tmp
wget http://ftp.cz.debian.org/debian/pool/main/d/dkms/dkms_2.6.1-4_all.deb
sudo dpkg -i dkms_2.6.1-4_all.deb
```

Clone the dahdi-linux repository:
```bash
git clone https://gitlab.com/xivo.solutions/dahdi-linux/
```

Move to the directory:
```bash
cd dahdi-linux/
```

→ **Make sure that you have the headers for your kernel installed.**
In /usr/src, there should be files regarding your kernel version, for instance:
- ```linux-headers-4.9.0-9-amd64```
- ```linux-headers-4.9.0-9-common```

On the XiVO standard installation, at the time I am writing this, the headers for your kernel are not installed:
```
You do not appear to have the sources for the 4.9.0-8-amd64 kernel installed.
When doing ls /usr/src, nothing is there.
uname -r: 4.9.0-8-amd64
dpkg --list | grep linux-image:
ii  linux-image-4.9.0-8-amd64        4.9.144-3.1
amd64        Linux 4.9 for 64-bit PCs
ii  linux-image-amd64        4.9+80+deb9u6
amd64        Linux for 64-bit PCs (meta-package)
```
You have to make sure the file ```/usr/src/linux-headers-<kernel_version>-<architecture>``` is there. If not, you have to download the sources.

→ If the sources aren’t there, DKMS won’t be able to do its job.

To correct the problem, type this:
```bash
sudo apt-get update
sudo apt-get install linux-headers-4.9.0-8-amd64
```
Do not forget to replace ```4.9.0-8``` by your kernel version, which you can get by typing ```uname -r```. If ```apt``` doesn't find the headers, you can download them directly from here: https://kernel.ubuntu.com/~kernel-ppa/mainline/.

**Warning:** The following build commands can be used for development. Especially the `make`, `generate_conf` and `make clean` commands. But the whole package build procedure is maintained in `xivo-build-tools` project.

**Warning:** The following three steps may not generate the same result. You may need to re-run the make, generate_conf and make clean sequence to be able to complete the build.

Build the modules before generating a config:
```bash
make
```

Generate a config:
```bash
sudo build_tools/dkms-helper generate_conf > dkms.conf
```

Clean the configuration:
```bash
make clean
```

Copy everything in usr:
```bash
sudo mkdir -p /usr/src/dahdi-linux-3.1.0
sudo cp -R . /usr/src/dahdi-linux-3.1.0
```

**DKMS Remove** (Done just in case)
```bash
sudo dkms remove -m dahdi-linux -v 3.1.0 --all
```

**DKMS Add**
```bash
sudo dkms add -m dahdi-linux -v 3.1.0
```

**DKMS Build**
```bash
sudo dkms build -m dahdi-linux -v 3.1.0
```

**DKMS Mkdeb**

For this command to work, you have to install the following packages: ```fakeroot```, ```debhelper```
```bash
sudo dkms mkdeb -m dahdi-linux -v 3.1.0 --source-only
```

The .deb file is in ```/var/lib/dkms/dahdi-linux/3.1.0/deb/```.
```bash
cd ..
cp /var/lib/dkms/dahdi-linux/3.1.0/deb/dahdi-linux-dkms_3.1.0_amd64.deb .
```

→ At this point, if everything went smoothly, the .deb package was generated.
On any machine, you can then proceed and install dahdi-linux DKMS:
```bash
sudo dpkg -i dahdi-linux-dkms_3.1.0_amd64.deb
```
